library(loadeR)
library(transformeR)
library(ggplot2)
library(RColorBrewer)
library(maps)

rm(list=ls()) ; graphics.off() ; gc()

#Directory information
setwd("~/Downloads/ETCCDI_aemet-5km/")
OUTPUTS <- "../OUTPUTS/"

#Useful information
ETCCDI_index <- c("Rx1day","Rx5day","TN10p","TN90p","TNn","TNx","TX10p","TX90p","TXn","TXx","CDD","CWD","CSDI",
                  "PRCPTOT","TR","R10mm","R20mm","R95pTOT","WSDI")
seasons <- list(Summer=c(6,7,8),Autumn=c(9,10,11),Winter=c(12,1,2),Spring=c(3,4,5))

##############---spatial plots---##############################################
#Load data
for(ind in 1:length(ETCCDI_index)){
  message(ETCCDI_index[ind])
  files <- list.files(pattern = paste(ETCCDI_index[ind],"_aemet-5km",sep=""))
  
  for(i in 1:length(files)){
    di <- dataInventory(dataset = files[i])
    nc <- loadGridData(dataset=files[i],var=names(di))
    
    if(di[[names(di)]][["Units"]]=="m s-1"){
      di[[names(di)]][["Units"]] <- "mm"
      palette=brewer.pal(n=11,name="YlGnBu")
    }
    
    if(di[[names(di)]][["Units"]]=="K"){
      palette=rev(brewer.pal(n=11,name="RdYlBu"))
      nc <- gridArithmetics(nc,273,operator = "-")
      di[[names(di)]][["Units"]] <- "ºC"
    }
    
    if(di[[names(di)]][["Units"]]=="percentage"){
      palette=rev(brewer.pal(n=11,name="RdYlBu"))
    }
    
    if(di[[names(di)]][["Units"]]=="days"){
      palette=brewer.pal(n=11,name="PuBuGn")
    }
    
    if(di[[names(di)]][["Units"]]=="mm"){
      palette=brewer.pal(n=9,name="YlGnBu")
    }
    
    ###############
    #Aggregation of annual/seasonal indices (climatology)
    data <- list()
    
    if(length(nc$Dates$start)<=72){
      index2 <- redim(climatology(nc),drop=T)
      data[[1]] <- data.frame(Index=as.numeric(index2$Data),
                              Longitude=rep(nc$xyCoords$x,each=length(nc$xyCoords$y)),
                              Latitude=rep(nc$xyCoords$y,length(nc$xyCoords$x)),
                              Season=rep("Annual",length(as.numeric(index2$Data))))
    }
    if(length(nc$Dates$start)>72){
      for(j in 1:length(seasons)){
        index2 <- redim(climatology(subsetGrid(nc,season = seasons[[j]])),drop=T)
        data[[j+1]] <- data.frame(Index=as.numeric(index2$Data),
                                  Longitude=rep(nc$xyCoords$x,each=length(nc$xyCoords$y)),
                                  Latitude=rep(nc$xyCoords$y,length(nc$xyCoords$x)),
                                  Season=rep(names(seasons)[j],length(as.numeric(index2$Data))))
      }}
  }
  
  #Organize data for ggplot plotting and saving
  datos <- do.call(rbind,data)
  datos$Season <- factor(datos$Season,levels=c("Winter","Autumn","Summer","Spring","Annual"))
  
  lims <- range(datos$Index,na.rm=T)
  png(filename = paste(OUTPUTS,ETCCDI_index[ind],"_aemet-5km_SpatialPattern_IberianPeninsula.png",sep=""),res=300,width=3000,height=1500)
  print(ggplot(data=datos,aes(x=Longitude,y=Latitude,fill=Index)) + geom_tile() + facet_wrap(~Season,nrow = 1) +
          borders("world",colour = "black",size=0.2) + scale_x_continuous(n.breaks = 6) + scale_y_continuous(n.breaks = 6) +
          coord_quickmap(xlim = range(datos$Longitude),ylim=range(datos$Latitude),expand = F) + 
          scale_color_gradientn(di[[names(di)]][["Units"]],colours = palette,
                                aesthetics = "fill",na.value = "white") +
          theme_bw() + ggtitle(paste("AEMET-5km",names(di),sep=" ")) )
  dev.off()
  
  png(filename = paste(OUTPUTS,ETCCDI_index[ind],"_aemet-5km_SpatialPattern_Catalunya.png",sep=""),res=300,width=3000,height=1500)
  print(ggplot(data=subset(datos,Latitude>40 & Latitude < 43 & Longitude > 0 & Longitude < 4),
               aes(x=Longitude,y=Latitude,fill=Index)) + geom_tile() + facet_wrap(~Season,nrow = 1) +
          borders("world",colour = "black",size=0.2) + scale_x_continuous(n.breaks = 6) + scale_y_continuous(n.breaks = 6) +
          coord_quickmap(ylim = c(40,43),xlim=c(0,4),expand = F) + 
          scale_color_gradientn(di[[names(di)]][["Units"]],colours = palette,
                                aesthetics = "fill",na.value = "white") +
          theme_bw() + ggtitle(paste("AEMET-5km",names(di),sep=" ")))
  dev.off()
}

##############---temporal plots---##############################################
# for(ind in 1:length(ETCCDI_index)){
#   #Load data
#   files <- list.files(pattern = ETCCDI_index[ind])
#   data <- list() ; data.cat <- list()
#   
#   for(i in 1:length(files)){
#     di <- dataInventory(dataset = files[i])
#     nc <- loadGridData(dataset=files[i],var=names(di))
#     
#     #Aggregation of annual/seasonal indices (climatology: time-series)
#     if(length(nc$Dates$start)<=72){
#       index2 <- aggregateGrid(nc,aggr.spatial = list(FUN="mean",na.rm=T))
#       index.cat <- aggregateGrid(subsetGrid(nc,lonLim = c(0,3.5),latLim = c(40,43)),
#                                  aggr.spatial = list(FUN="mean",na.rm=T))
#       
#       data[[1]] <- data.frame(Index=as.numeric(index2$Data),
#                               Season=rep("Annual",length(as.numeric(index2$Data))),
#                               Years=getYearsAsINDEX(index2))
#       data.cat[[1]] <- data.frame(Index=as.numeric(index.cat$Data),
#                                   Season=rep("Annual",length(as.numeric(index.cat$Data))),
#                                   Years=getYearsAsINDEX(index.cat))
#     }
#     if(length(nc$Dates$start)>72){
#       for(j in 1:length(seasons)){
#         index2 <- redim(aggregateGrid(nc,aggr.s = list(FUN=list("mean",na.rm=T),season=seasons[[j]])),drop=T)
#         index.cat <- subsetGrid(index2,lonLim = c(0,3.5),latLim = c(40,43))
#         
#         index2 <- aggregateGrid(index2,aggr.spatial = list(FUN="mean",na.rm=T))
#         index.cat <- aggregateGrid(index.cat,aggr.spatial = list(FUN="mean",na.rm=T))
#         
#         data.cat[[j+1]] <- data.frame(Index=as.numeric(index.cat$Data),
#                                       Season=rep(names(seasons)[j],length(as.numeric(index.cat$Data))),
#                                       Years=getYearsAsINDEX(index.cat))
#         
#         data[[j+1]] <- data.frame(Index=as.numeric(index2$Data),
#                                   Season=rep(names(seasons)[j],length(as.numeric(index2$Data))),
#                                   Years=getYearsAsINDEX(index2))
#       }}
#   }
#   
#   #Organize for ggplot plotting and saving
#   datos <- do.call(rbind,data)
#   datos$Season <- factor(datos$Season,levels=c("Winter","Autumn","Summer","Spring","Annual"))
#   
#   datos.cat <- do.call(rbind,data.cat)
#   datos.cat$Season <- factor(datos.cat$Season,levels=c("Winter","Autumn","Summer","Spring","Annual"))
#   
#   mix <- rbind(datos,datos.cat)
#   mix$Region <- rep(c("Iberian Peninsula","Catalunya"),each=dim(datos)[1])
#   
#   if(length(unique(mix$Season))>1){
#     mix <- mix[-which(mix$Season=="Annual"),]
#   }
#   
#   # png(filename = paste(OUTPUTS,ETCCDI_index[ind],"_E-OBS_TimeSeries_IberianPeninsula.png",sep=""),res=300,width=3000,height=1500)
#   # print(ggplot(data=datos,aes(x=Years,y=Index)) + geom_line() + facet_wrap(~Season,nrow = 3) +
#   #         scale_x_continuous(n.breaks = 6) + scale_y_continuous(n.breaks = 6) +
#   #         theme_bw() + ggtitle(paste("E-OBS",names(di),sep=" ")) + geom_smooth(method=lm,se=F) ) 
#   # dev.off()
#   # 
#   # png(filename = paste(OUTPUTS,ETCCDI_index[ind],"_E-OBS_TimeSeries_Catalunya.png",sep=""),res=300,width=3000,height=1500)
#   # print(ggplot(data=datos.cat,aes(x=Years,y=Index)) + geom_line() + facet_wrap(~Season,nrow = 3) +
#   #         scale_x_continuous(n.breaks = 6) + scale_y_continuous(n.breaks = 6) +
#   #         theme_bw() + ggtitle(paste("E-OBS",names(di),sep=" ")) + geom_smooth(method=lm,se=F) ) 
#   # dev.off()
  
  # png(filename = paste(OUTPUTS,ETCCDI_index[ind],"_E-OBS_TimeSeries_Comparison.png",sep=""),res=300,width=3000,height=1500)
  # print(ggplot(data=mix,aes(x=Years,y=Index,color=Region)) + geom_line() + facet_wrap(~Season,nrow = 3) +
  #         scale_x_continuous(n.breaks = 6) + scale_y_continuous(n.breaks = 6) +
  #         theme_bw() + ggtitle(paste("E-OBS",names(di),sep=" ")) + geom_smooth(method=lm,se=F)) 
  # dev.off()
} 
